@@ -51,6 +51,14 @@ except ImportError:
 lg = logging.getLogger(__name__)
 lg.setLevel(logging.DEBUG)
 
+
+#thanks: http://tinyurl.com/mznq746
+class ArgumentParserError(Exception): pass
+class ThrowingArgumentParser(argparse.ArgumentParser):
+    def error(self, message):
+        raise ArgumentParserError(message)
+
+
 # hack to quickly get the verbosity set properly:
 # if '-v' in sys.argv:
 #     lg.setLevel(logging.DEBUG)
@@ -131,6 +139,7 @@ def get_config(name,
 
 
 def set_local_config(app, key, value):
+
     app.conf[key] = value
     with open(get_conf_pickle_location(app.name), 'wb') as F:
         pickle.dump(app.conf, F)
@@ -150,6 +159,17 @@ def get_local_config_filename(name):
         os.path.expanduser('~'),
         '.config', name, 'config', '_local.config')
 
+def get_cache_dir(name, *args):
+
+    cd = os.path.join(
+            os.path.expanduser('~'),
+            '.cache', name)
+    if args:
+        cd = os.path.join(cd, *args)
+
+    if not os.path.exists(cd):
+        os.makedirs(cd)
+    return cd
 
 def get_local_config_file(name):
     fn = get_local_config_filename(name)
@@ -227,6 +247,8 @@ class app(object):
 
         self.config_files = config_files
 
+        self.leip_on_parse_error = None
+
         self.leip_commands = {}
         self.leip_subparsers = {}
         self.plugins = {}
@@ -235,7 +257,7 @@ class app(object):
         self.hooks = defaultdict(list)
 
         if not disable_commands:
-            self.parser = argparse.ArgumentParser()
+            self.parser = ThrowingArgumentParser()
 
             self.parser.add_argument('-v', '--verbose', action='store_true')
             self.parser.add_argument('-q', '--quiet', action='store_true')
@@ -283,9 +305,10 @@ class app(object):
                 module_name))
             try:
                 mod = importlib.import_module(module_name)
-            except ImportError:
+            except ImportError as e:
                 lg.error('Cannot import %s module "%s" (%s)',
                          self.name, plugin_name, module_name)
+                lg.error('%s', e)
                 continue
 
             self.plugins[plugin_name] = mod
@@ -331,7 +354,25 @@ class app(object):
 
         # register parse arguments as a hook
         def _prep_args(app):
-            args = self.parser.parse_args()
+            try:
+                args = self.parser.parse_args()
+            except ArgumentParserError as e:
+                #invalid call - see if there is an overriding function
+                #to capture invalid calls
+                if app.leip_on_parse_error is None:
+                    super(ThrowingArgumentParser,
+                          self.parser).error(e.message)
+                else:
+                    lg.debug("parse error: %s", e.message)
+                    rv = app.leip_on_parse_error(app, e)
+
+                    #if the on_parse_error function returns a non-zero
+                    #number the error is raised after all
+                    if rv != 0:
+                        super(ThrowingArgumentParser,
+                              self.parser).error(e.message)
+                    exit(0)
+            #     raise
             self.trans['args'] = args
             rootlogger = logging.getLogger()
             if self.trans['args'].verbose:
@@ -349,6 +390,12 @@ class app(object):
         # discover locally
         self.discover(globals())
 
+    def cache_dir(self, group=None):
+        """
+        Return a (created) cache directory for this app
+        """
+        return get_cache_dir(self.name, group)
+
     def discover(self, mod):
         """
         discover all hooks & commands in the provided module or
@@ -377,6 +424,7 @@ class app(object):
         """
 
         leip_init_hook = None
+
         for obj_name in mod_objects:
             obj = mod_objects[obj_name]
 
@@ -410,7 +458,12 @@ class app(object):
             if not hasattr(obj, '__call__'):
                 continue
 
+
             # see if this is a hook
+            if hasattr(obj, '__call__') and \
+                    hasattr(obj, '_leip_parse_error_hook'):
+                self.leip_on_parse_error = obj
+
             if hasattr(obj, '_leip_hook'):
                 hook = obj._leip_hook
                 if isinstance(hook, fantail.Fantail):
@@ -512,7 +565,7 @@ class app(object):
         Execute hook
         """
         to_run = sorted(self.hooks[name])
-        lg.debug("running hook %s" % name)
+        lg.debug("starting hook %s" % name)
 
         for priority, func in to_run:
             #print(priority, func)
@@ -521,7 +574,7 @@ class app(object):
 
     def run(self):
         for hook in self.hook_order:
-            lg.debug("running hook {}".format(hook))
+            lg.debug("running hook {}/{}".format(self.name, hook))
             self.run_hook(hook)
 
 
@@ -625,6 +678,15 @@ def init(f):
     f._leip_init_hook = f.__name__
     return f
 
+def on_parse_error(f):
+    """
+    Execute this function on a parse_error
+
+    Only one per applications is expected.
+    """
+    f._leip_parse_error_hook = f.__name__
+    return f
+
 
 #
 # Hook decorators
